@@ -66,6 +66,58 @@ impl Display for ParseHumanReadableDurationError {
 
 impl std::error::Error for ParseHumanReadableDurationError {}
 
+/// The internally used time units which are supported.
+enum InternalTimeUnit {
+    Seconds,
+    Minutes,
+}
+
+impl FromStr for InternalTimeUnit {
+    type Err = ();
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        // ensure that the string has to be at least one character long
+        if s.len() < 1 {
+            return Err(());
+        }
+
+        // match the first character to the corresponding unit
+        match s.to_lowercase().chars().next().unwrap() {
+            's' => Ok(InternalTimeUnit::Seconds),
+            'm' => Ok(InternalTimeUnit::Minutes),
+            _ => Err(()),
+        }
+    }
+}
+
+/// A tuple of a time unit and the corresponding value (only for internal use).
+struct InternalTime(u64, InternalTimeUnit);
+
+/// A method for extracting the containing time information from a string. This method should
+/// only be used internally.
+fn extract_time_information(value: &str) -> Vec<InternalTime> {
+    use lazy_static::lazy_static;
+    use regex::Regex;
+
+    // compile the regular expression for extracting the supported timings
+    lazy_static! {
+        static ref TIME_REGEX: Regex = Regex::from_str(r"([0-9]+)([ms]){1}").unwrap();
+    }
+
+    // collect all found matches
+    let mut found_matches = vec![];
+    for capture in TIME_REGEX.captures_iter(value) {
+        if let Ok(time) = u64::from_str(&capture[1]) {
+            if let Ok(unit) = InternalTimeUnit::from_str(&capture[2]) {
+                found_matches.push(InternalTime(time, unit))
+            }
+        }
+    }
+
+    // return the found matches
+    found_matches
+}
+
 /// Parse a value from a string
 ///
 /// `FromStr`'s `from_str` method is often used implicitly, through
@@ -90,25 +142,25 @@ impl FromStr for HumanReadableDuration {
     /// assert_eq!(50, x.seconds());
     /// ```
     fn from_str(value: &str) -> Result<Self, Self::Err> {
-        if value.contains('m') {
-            let actual_time_value = value.split("m").next().unwrap().trim();
-            let maybe_parsed_time_value = actual_time_value.parse::<u64>();
-            if maybe_parsed_time_value.is_ok() {
-                return Ok(HumanReadableDuration {
-                    time_in_seconds: maybe_parsed_time_value.unwrap() * 60,
-                });
+        // try to get the time information from the passed string
+        let time_information = extract_time_information(value);
+
+        // if we could not extract any information, return an error
+        if time_information.is_empty() {
+            return Err(ParseHumanReadableDurationError);
+        }
+
+        // sum up the seconds and return corresponding object
+        let mut seconds = 0;
+        for current_time_object in time_information {
+            match current_time_object.1 {
+                InternalTimeUnit::Seconds => seconds += current_time_object.0,
+                InternalTimeUnit::Minutes => seconds += current_time_object.0 * 60,
             }
         }
-        if value.contains('s') {
-            let actual_time_value = value.split("s").next().unwrap().trim();
-            let maybe_parsed_time_value = actual_time_value.parse::<u64>();
-            if maybe_parsed_time_value.is_ok() {
-                return Ok(HumanReadableDuration {
-                    time_in_seconds: maybe_parsed_time_value.unwrap(),
-                });
-            }
-        }
-        Err(ParseHumanReadableDurationError)
+        return Ok(HumanReadableDuration {
+            time_in_seconds: seconds,
+        });
     }
 }
 
@@ -220,11 +272,9 @@ mod tests {
     }
 
     #[test]
-    fn from_str_10_s_works() {
+    fn from_str_10_s_will_be_handled_gracefully() {
         let representation = HumanReadableDuration::from_str("10 s");
-        assert_eq!(true, representation.is_ok());
-        assert_eq!(10, representation.as_ref().unwrap().seconds());
-        assert_eq!(0, representation.as_ref().unwrap().minutes());
+        assert_eq!(true, representation.is_err());
     }
 
     #[test]
@@ -252,11 +302,9 @@ mod tests {
     }
 
     #[test]
-    fn from_str_5_m_works() {
+    fn from_str_5_m_will_be_handled_gracefully() {
         let representation = HumanReadableDuration::from_str("5 m");
-        assert_eq!(true, representation.is_ok());
-        assert_eq!(300, representation.as_ref().unwrap().seconds());
-        assert_eq!(5, representation.as_ref().unwrap().minutes());
+        assert_eq!(true, representation.is_err());
     }
 
     #[test]
@@ -282,4 +330,42 @@ mod tests {
         assert_eq!(3660, representation.as_ref().unwrap().seconds());
         assert_eq!(61, representation.as_ref().unwrap().minutes());
     }
+
+    #[test]
+    fn from_str_4_m_10_s_will_be_handled_gracefully() {
+        let representation = HumanReadableDuration::from_str("4 m 10 s");
+        assert_eq!(true, representation.is_err());
+    }
+
+    #[test]
+    fn from_str_4m_10s_works() {
+        let representation = HumanReadableDuration::from_str("4m 10s");
+        assert_eq!(true, representation.is_ok());
+        assert_eq!(250, representation.as_ref().unwrap().seconds());
+        assert_eq!(4, representation.as_ref().unwrap().minutes());
+    }
+
+    #[test]
+    fn from_str_4m10s_works() {
+        let representation = HumanReadableDuration::from_str("4m10s");
+        assert_eq!(true, representation.is_ok());
+        assert_eq!(250, representation.as_ref().unwrap().seconds());
+        assert_eq!(4, representation.as_ref().unwrap().minutes());
+    }
+
+    #[test]
+    fn from_str_3m60s_works() {
+        let representation = HumanReadableDuration::from_str("3m60s");
+        assert_eq!(true, representation.is_ok());
+        assert_eq!(240, representation.as_ref().unwrap().seconds());
+        assert_eq!(4, representation.as_ref().unwrap().minutes());
+    }
+
+    #[test]
+    fn from_str_3m61s_works() {
+        let representation = HumanReadableDuration::from_str("3m61s");
+        assert_eq!(true, representation.is_ok());
+        assert_eq!(241, representation.as_ref().unwrap().seconds());
+        assert_eq!(4, representation.as_ref().unwrap().minutes());
+    }
 }
